# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_files": "00_utils.core.ipynb",
         "group_monthly": "01_utils.pandas_hacks.ipynb",
         "reindex_monthly_en": "01_utils.pandas_hacks.ipynb",
         "monthly": "01_utils.pandas_hacks.ipynb",
         "DataFrame.normalize": "01_utils.pandas_hacks.ipynb",
         "Series.normalize": "01_utils.pandas_hacks.ipynb",
         "DataFrame.min_max": "01_utils.pandas_hacks.ipynb",
         "Series.min_max": "01_utils.pandas_hacks.ipynb",
         "All": "01_utils.pandas_hacks.ipynb",
         "DataFrame.flatten_cols": "01_utils.pandas_hacks.ipynb",
         "mem_usage": "01_utils.pandas_hacks.ipynb",
         "reduce_mem_usage": "01_utils.pandas_hacks.ipynb",
         "read_mat": "03_utils.matlab_io.ipynb",
         "datetime_mtopy": "03_utils.matlab_io.ipynb",
         "get_components": "03_utils.matlab_io.ipynb",
         "dict2tuples": "03_utils.matlab_io.ipynb",
         "make_datetime_index": "03_utils.matlab_io.ipynb",
         "mat2df": "03_utils.matlab_io.ipynb",
         "read_pvgis": "03_utils.tmy.ipynb",
         "read_tmy": "03_utils.tmy.ipynb",
         "is_day": "04_utils.missing.ipynb",
         "plot_missing": "04_utils.missing.ipynb",
         "std_outliers": "04_utils.missing.ipynb",
         "diff_outliers": "04_utils.missing.ipynb",
         "comp": "04_utils.missing.ipynb",
         "view_anomaly": "04_utils.missing.ipynb",
         "stuck_outlier": "04_utils.missing.ipynb",
         "percentile_cutoff": "04_utils.missing.ipynb",
         "MinMaxRemoval": "04_utils.missing.ipynb",
         "plot2series": "05_utils.plot.ipynb",
         "add_value_labels": "05_utils.plot.ipynb",
         "plot_yearly_gain": "05_utils.plot.ipynb",
         "CHAMBERY": "06_modelling.mybifacialvf.ipynb",
         "rename_cols": "06_modelling.mybifacialvf.ipynb",
         "format_output_as_pvfactor": "06_modelling.mybifacialvf.ipynb",
         "run_bifacialvf_simulation": "06_modelling.mybifacialvf.ipynb",
         "system_def": "07_modelling.mypvfactors.ipynb",
         "get_index": "07_modelling.mypvfactors.ipynb",
         "plot_idx": "07_modelling.mypvfactors.ipynb",
         "fit_and_plot": "07_modelling.mypvfactors.ipynb",
         "join_sun": "07_modelling.mypvfactors.ipynb",
         "get_data": "07_modelling.mypvfactors.ipynb",
         "fn_build_report": "07_modelling.mypvfactors.ipynb",
         "run_pvfactors_simulation": "07_modelling.mypvfactors.ipynb",
         "individual_report": "07_modelling.mypvfactors.ipynb"}

modules = ["utils/core.py",
           "utils/pandas_hacks.py",
           "utils/matlab_io.py",
           "utils/tmy.py",
           "utils/missing.py",
           "utils/plot.py",
           "modelling/mybifacialvf.py",
           "modelling/mypvfactors.py"]

doc_url = "https://tcapelle.github.io/pvtools/"

git_url = "https://github.com/tcapelle/pvtools/tree/master/"

def custom_doc_links(name): return None
